// Model : récupération et mise à jour des données
let ArticleModel = {
  list: [],
  loadList: function() {
    return m.request({
      method: "GET",
      url: "/articles"
    })
    .then(res => {
      // res.articles: array of {id, parentId, title, content, created, likes}
      ArticleModel.list = res.articles
        .filter(p => !p.parentId)
        .map(p => {
          return Object.assign(
            {comments: res.articles.filter(c => c.parentId == p.id)},
            p
          )
        })
    })
  }
}
// Update data structure + re-render after SSE
const eventSource = new EventSource('/updates')
eventSource.onmessage = (msg) => {
  let o = JSON.parse(msg.data)
  const code = o.code
  delete o["code"]
  switch (code) {
    case "new_post":
      o.comments = []
      ArticleModel.list.push(o)
      break
    case "new_comment":
      ArticleModel.list.find(p => p.id == o.parentId).comments.push(o)
      break
    case "delete":
      if (!o.parentId) {
        ArticleModel.list.splice(
          ArticleModel.list.findIndex(p => p.id == o.id), 1)
      }
      else {
        let parentPost = ArticleModel.list.find(p => p.id == o.parentId)
        parentPost.comments.splice(
          parentPost.comments.findIndex(c => c.id == o.id), 1)
      }
      break
  }
  m.redraw()
}

// View : structure de la page, présentation des données
let ArticleView = {
  newTitle: m.stream(""),
  newContent: m.stream(""),
  oninit: ArticleModel.loadList,
  view: function() {
    return m(".article-list",
      ArticleModel.list.map(p => {
        return m(".post", [
          m("h4", [
            m("span", p.title),
            m("span.date", new Date(p.created).toISOString().slice(0,-5)),
            m("span.cross",
              {onclick: () => ServerUpdate.remove(p.id)},
              m.trust("&#10006;")),
          ]),
          m("p", p.content),
          m(".comments", p.comments.map(c => {
            return m(".comment", [
              m("h4", [
                m("span.date", new Date(c.created).toISOString().slice(0,-5)),
                m("span.cross",
                  {onclick: () => ServerUpdate.remove(c.id, p.id)},
                  m.trust("&#10006;"))
              ]),
              m("p", c.content)
            ])
          })),
          m(`.comm_form#comm_${p.id}`, [
            m("textarea[placeholder=Contenu]"),
            m("button",
              {onclick: () => ServerUpdate.addComment(p.id)},
              "Commenter")
          ])
        ])
      }).concat(
      m(".post_form", [
        m("input[type=text][placeholder=Titre]",
          {onchange: (e) => ArticleView.newTitle(e.target.value)}),
        m("textarea[placeholder=Contenu]",
          {onchange: (e) => ArticleView.newContent(e.target.value)}),
        m("button",
          {onclick: () => {
            const [title, content] = [ArticleView.newTitle, ArticleView.newContent]
            ArticleView.newTitle = ArticleView.newContent = ""
            ServerUpdate.addPost(title, content)
          }},
          "Poster")
      ]))
    )
  }
}
// Actions utilisateur = mise à jour côté serveur (DB)
const ServerUpdate = {
  addPost: function(title, content) {
    m.request({
      method: "POST",
      url: "/posts",
      body: {
        title: title,
        content: content
      }
    })
  },
  addComment: function(parentId) {
    let comment_txt = document.querySelector(`#comm_${parentId} > textarea`)
    m.request({
      method: "POST",
      url: "/comments",
      body: {
        parentId: parentId,
        content: comment_txt.value
      }
    })
    comment_txt.value = ""
  },
  remove: function(id, parentId) {
    let url = `/articles?id=${id}`
    if (parentId) url += `&parentId=${parentId}`
    m.request({
      method: "DELETE",
      url: url
    })
  }
}

m.mount(document.body, ArticleView)
