const sqlite3 = require('sqlite3').verbose();
const db = new sqlite3.Database('db.sqlite')

const express = require('express')
const app = express()
app.use(express.json()) //for POST body in req
app.use(express.static('public'))
app.get('/', (req,res) => {
  res.sendFile(__dirname + '/index.html')
})

const events = require('events')
const myEmitter = new events.EventEmitter();

app.get('/updates', (req, res) => {
  res.set({
    'Content-Type': 'text/event-stream',
    'Cache-Control': 'no-cache',
    'Connection': 'keep-alive'
  })
  res.flushHeaders()
  console.log("connected")
  // NOTE: need functions handlers for removeListener(), so cannot factorize
  const jcd = (code, data) => JSON.stringify(Object.assign({code: code}, data))
  const writePost =
    (data) => res.write("data: " + jcd("new_post", data) + "\n\n")
  const writeComm =
    (data) => res.write("data: " + jcd("new_comment", data) + "\n\n")
  const writeDel =
    (data) => res.write("data: " + jcd("delete", data) + "\n\n")
  myEmitter.on('new_post', writePost)
  myEmitter.on('new_comment', writeComm)
  myEmitter.on('delete', writeDel)
  req.on('close', function(err) {
    myEmitter.removeListener("new_post", writePost)
    myEmitter.removeListener("new_comment", writeComm)
    myEmitter.removeListener("delete", writeDel)
  })
})

app.get("/articles", (req,res) => {
  db.serialize(function() {
    db.all("SELECT * FROM Articles", (err,ret) => {
      res.json({articles: ret})
    })
  })
})

app.delete("/articles", (req,res) => {
  db.serialize(function() {
    db.run("DELETE FROM Articles WHERE id=?", req.query.id);
  })
  myEmitter.emit('delete', req.query)
})

app.post("/posts", (req,res) => {
  let new_post = {
    title: req.body.title,
    content: req.body.content,
    created: Date.now()
  }
  db.serialize(function() {
    db.run("INSERT INTO Articles (title,content,created) VALUES (?,?,?) ",
           Object.values(new_post))
    db.get("SELECT last_insert_rowid() as lastid", (err,ret) => {
      new_post.id = ret.lastid
      myEmitter.emit('new_post', new_post)
    })
  })
})

app.post("/comments", (req,res) => {
  let new_comment = {
    parentId: req.body.parentId,
    content: req.body.content,
    created: Date.now()
  }
  db.serialize(function() {
    db.run("INSERT INTO Articles (parentId,content,created) VALUES (?,?,?) ",
           Object.values(new_comment))
    db.get("SELECT last_insert_rowid() as lastid", (err,ret) => {
      new_comment.id = ret.lastid
      myEmitter.emit('new_comment', new_comment)
    })
  })
})

const port = 3001
app.listen(port, () => {
  console.log(`App listening at http://localhost:${port}`)
})
